searchData={"items":[{"type":"module","title":"Sham","doc":"Sham is a mock HTTP(S) server useful for testing HTTP(S) clients.","ref":"Sham.html"},{"type":"function","title":"Sham.expect/2","doc":"Expect at least one call to the Sham instance using any method and at any path.","ref":"Sham.html#expect/2"},{"type":"function","title":"Examples - Sham.expect/2","doc":"iex> sham = Sham.start()\n    iex> Sham.expect(sham, fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"Hello\"}\n    iex> ShamTest.post(\"http://localhost:#{sham.port}/other\", \"foo=bar\")\n    {:ok, 200, \"Hello\"}\n\n    sham = Sham.start()\n    Sham.expect(sham, fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    ** (ExUnit.AssertionError) No HTTP request was received by Sham\n\nIf no request is sent to the sham port, an assertion error will be raised.","ref":"Sham.html#expect/2-examples"},{"type":"function","title":"Sham.expect/4","doc":"Expect at least one call to the Sham instance using the given method and path.\n\n- `method` - The HTTP method to expect. Should be one of `\"GET\"`, `\"POST\"`, `\"PUT\"`, `\"PATCH\"`, `\"DELETE\"`, `\"HEAD\"`, or `\"OPTIONS\"` or (Anything Plug supports).\n- `path` - The path to expect","ref":"Sham.html#expect/4"},{"type":"function","title":"Examples - Sham.expect/4","doc":"iex> sham = Sham.start()\n    iex> Sham.expect(sham, \"GET\", \"/hello\", fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"Hello\"}\n\n    # Sending a request that does not match an expectation will result in an assertion error\n    ShamTest.post(\"http://localhost:#{sham.port}/other\", \"foo=bar\")\n    {:ok, 500, \"Unexpected request to Sham: POST /other\"}\n    ** (ExUnit.AssertionError) Unexpected request to Sham: POST /other\n\nIf no request is sent to the sham port using the given method and path, an assertion error will be raised.","ref":"Sham.html#expect/4-examples"},{"type":"function","title":"Sham.expect_none/1","doc":"Expect no requests to the Sham instance using any method or path.","ref":"Sham.html#expect_none/1"},{"type":"function","title":"Examples - Sham.expect_none/1","doc":"sham = Sham.start()\n    Sham.expect_none(sham)\n    # Sending a request will result in an assertion error\n    ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 500, \"A request was received by Sham when none were expected: GET /hello\"}\n    ** (ExUnit.AssertionError) A request was received by Sham when none were expected: GET /hello","ref":"Sham.html#expect_none/1-examples"},{"type":"function","title":"Sham.expect_none/3","doc":"Expect no requests to the Sham instance for a given method and path.\n\n- `method` - The HTTP method to expect. Should be one of `\"GET\"`, `\"POST\"`, `\"PUT\"`, `\"PATCH\"`, `\"DELETE\"`, `\"HEAD\"`, or `\"OPTIONS\"` or (Anything Plug supports).\n- `path` - The path to expect","ref":"Sham.html#expect_none/3"},{"type":"function","title":"Sham.expect_once/2","doc":"Expect exactly one call to the Sham instance using any method and at any path.\n\nYou can stack multiple calls to `expect_once/4` to expect multiple requests.","ref":"Sham.html#expect_once/2"},{"type":"function","title":"Examples - Sham.expect_once/2","doc":"iex> sham = Sham.start()\n    iex> Sham.expect_once(sham, fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"Hello\"}\n\n    iex> sham = Sham.start()\n    iex> Sham.expect_once(sham, fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"Hello\"}\n    iex> Sham.expect_once(sham, fn conn -> Plug.Conn.send_resp(conn, 200, \"World\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"World\"}\n\n    # Sending a more than one request will result in an assertion error\n    ShamTest.post(\"http://localhost:#{sham.port}/other\", \"foo=bar\")\n    {:ok, 500, \"Exceeded expected requests to Sham: POST /other\"}\n    ** (ExUnit.AssertionError) Exceeded expected requests to Sham: POST /other\n\nIf a request is sent to the sham port using a different method or path, an assertion error will be raised.","ref":"Sham.html#expect_once/2-examples"},{"type":"function","title":"Sham.expect_once/4","doc":"Expect exactly one call to the Sham instance using the given method and path.\n\nYou can stack multiple calls to `expect_once/4` to expect multiple requests.\n\n- `method` - The HTTP method to expect. Should be one of `\"GET\"`, `\"POST\"`, `\"PUT\"`, `\"PATCH\"`, `\"DELETE\"`, `\"HEAD\"`, or `\"OPTIONS\"` or (Anything Plug supports).\n- `path` - The path to expect","ref":"Sham.html#expect_once/4"},{"type":"function","title":"Examples - Sham.expect_once/4","doc":"iex> sham = Sham.start()\n    iex> Sham.expect_once(sham, \"GET\", \"/hello\", fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"Hello\"}\n\n    iex> sham = Sham.start()\n    iex> Sham.expect_once(sham, \"GET\", \"/hello\", fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"Hello\"}\n    iex> Sham.expect_once(sham, \"GET\", \"/hello\", fn conn -> Plug.Conn.send_resp(conn, 200, \"World\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"World\"}\n\n    # Sending a more than one request with the same method and path will result in an assertion error\n    ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 500, \"Exceeded expected requests to Sham: GET /hello\"}\n    ** (ExUnit.AssertionError) Exceeded expected requests to Sham: GET /hello\n\nIf no request is sent to the sham port using the given method and path, an assertion error will be raised.","ref":"Sham.html#expect_once/4-examples"},{"type":"function","title":"Sham.pass/1","doc":"Forces the Sham instance to pass regardless of whether a valid request was sent to the sham port or not.","ref":"Sham.html#pass/1"},{"type":"function","title":"Examples - Sham.pass/1","doc":"iex> sham = Sham.start()\n    iex> Sham.expect(sham, \"GET\", \"/hello\", fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.post(\"http://localhost:#{sham.port}/other\", \"foo=bar\")\n    {:ok, 500, \"Unexpected request to Sham: POST /other\"}\n    iex> Sham.pass(sham)\n\n    iex> sham = Sham.start()\n    iex> Sham.expect(sham, \"GET\", \"/hello\", fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> Sham.pass(sham)\n\nNo assertions will be raised.","ref":"Sham.html#pass/1-examples"},{"type":"function","title":"Sham.start/1","doc":"Starts a new Sham instance.","ref":"Sham.html#start/1"},{"type":"function","title":"Options - Sham.start/1","doc":"* `:ssl` - Whether to start the server in SSL mode. Defaults to `false`.\n* `:keyfile` - The path to an SSL keyfile. Defaults to an internal self-signed key.\n* `:certfile` - The path to an SSL certfile. Defaults to an internal self-signed certificate.","ref":"Sham.html#start/1-options"},{"type":"function","title":"Examples - Sham.start/1","doc":"sham = Sham.start()\n    Sham.expect(sham, fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n\n    sham = Sham.start(ssl: true)\n    Sham.expect(sham, fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)","ref":"Sham.html#start/1-examples"},{"type":"function","title":"Sham.stub/2","doc":"Provide a callback to handle any requests to the Sham instance without assertions.\n\nNo exceptions will be raised if no requests are sent to the sham port.","ref":"Sham.html#stub/2"},{"type":"function","title":"Examples - Sham.stub/2","doc":"iex> sham = Sham.start()\n    iex> Sham.stub(sham, fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"Hello\"}\n    iex> ShamTest.post(\"http://localhost:#{sham.port}/other\", \"foo=bar\")\n    {:ok, 200, \"Hello\"}","ref":"Sham.html#stub/2-examples"},{"type":"function","title":"Sham.stub/4","doc":"Provide a callback to handle any requests to the Sham instance using the given method and path without assertions.\n\nNo exceptions will be raised if no requests are sent to the sham port using the given method and path.","ref":"Sham.html#stub/4"},{"type":"function","title":"Examples - Sham.stub/4","doc":"iex> sham = Sham.start()\n    iex> Sham.stub(sham, \"GET\", \"/hello\", fn conn -> Plug.Conn.send_resp(conn, 200, \"Hello\") end)\n    iex> ShamTest.get(\"http://localhost:#{sham.port}/hello\")\n    {:ok, 200, \"Hello\"}\n\n    # Sending a request with a different method and path will result in an assertion error\n    ShamTest.post(\"http://localhost:#{sham.port}/other\", \"foo=bar\")\n    {:ok, 500, \"Unexpected request to Sham: POST /other\"}\n    ** (ExUnit.AssertionError) Unexpected request to Sham: POST /other","ref":"Sham.html#stub/4-examples"},{"type":"type","title":"Sham.sham_opts/0","doc":"","ref":"Sham.html#t:sham_opts/0"},{"type":"type","title":"Sham.t/0","doc":"","ref":"Sham.html#t:t/0"},{"type":"extras","title":"Sham","doc":"# Sham\n\nAn Elixir mock HTTP(S) server useful for testing HTTP(S) clients.\n\n![example workflow](https://github.com/andrewtimberlake/sham/actions/workflows/main.yml/badge.svg)\n[![Hex version badge](https://img.shields.io/hexpm/v/sham.svg)](https://hex.pm/packages/sham)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/sham/)\n[![License badge](https://img.shields.io/hexpm/l/sham.svg)](https://github.com/andrewtimberlake/sham/blob/master/LICENSE)\n\nDocumentation is available at  .","ref":"readme.html"},{"type":"extras","title":"Installation - Sham","doc":"The package can be installed by adding `sham` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:sham, \"~> 1.0\", only: :test}\n  ]\nend\n```\n\nIf you’re not using Sham in a web project, you’ll also need to add either `:plug_cowboy` or `:bandit` to your test dependencies.\n\n```elixir\ndef test_deps do\n  [\n    {:plug_cowboy, \"~> 2.0\", only: :test},\n    {:bandit, \"~> 1.0\", only: :test}\n  ]\nend\n```\n\nSham should pick up on the correct server automatically, but if it doesn't, you can specify it manually.\n\n```elixir\nconfig :sham, server: :plug_cowboy\n# or\nconfig :sham, server: :bandit\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Sham","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic usage - Sham","doc":"```elixir\n  sham = Sham.start()\n\n  Sham.expect(sham, \"GET\", \"/\", fn conn ->\n    Plug.Conn.resp(conn, 200, \"Hello world\")\n  end)\n\n  {:ok, 200, response_body} = HttpClient.get(\"http://localhost:#{sham.port}\")\n\n  assert response_body == \"Hello world\"\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"SSL - Sham","doc":"```elixir\n  sham = Sham.start(ssl: true)\n  # or, optionally with your own certificate and key\n  # sham = Sham.start(ssl: true, certfile: \"/path/to/cert.pem\", keyfile: \"/path/to/key.pem\")\n\n  Sham.expect(sham, \"GET\", \"/\", fn conn ->\n    Plug.Conn.resp(conn, 200, \"Hello world\")\n  end)\n\n  {:ok, 200, response_body} = HttpClient.get(\"https://localhost:#{sham.port}\")\n\n  assert response_body == \"Hello world\"\n```","ref":"readme.html#ssl"},{"type":"extras","title":"Assertions in the expectation callback - Sham","doc":"While you can place assertions in the expectation callback, they will only surface in the test run if there are no failed assertions in the rest of the test.\nSham catches any assertion errors, returns a 500 response with the error and then re-raises the assertion in an on_exit callback which is only run after the test has finished.\n\n```elixir\nSham.expect(sham, \"GET\", \"/\", fn conn ->\n  # This assertion will fail because the request path is not /wrong\n  # Sham will return the assertion error in a 500 response and re-raise it in an on_exit callback.\n  assert conn.request_path == \"/wrong\"\n  Plug.Conn.resp(conn, 200, \"Hello world\")\nend)\n\n# This assertion will fail because the result from the expection will be a 500 response with the assertion error and the test will fail without the assertion from the expectation callback.\nassert {:ok, 200, \"Hello world\"} = HttpClient.get(\"https://localhost:#{sham.port}\")\n\n# This will work because there is no assertion here so the assertion from the expectation callback will be surfaced at the end of the test.\n# The test will fail with the assertion error from the expectation callback.\nHttpClient.get(\"https://localhost:#{sham.port}\")\n```","ref":"readme.html#assertions-in-the-expectation-callback"},{"type":"extras","title":"Accessing the body of a request - Sham","doc":"By default, Plug does not parse the body of a request. If you need to access the body of a request, you will need to manually call","ref":"readme.html#accessing-the-body-of-a-request"},{"type":"extras","title":"JSON body - Sham","doc":"```elixir\nSham.expect(sham, \"POST\", \"/\", fn conn ->\n  conn =\n    Plug.Parsers.call(\n      conn,\n      Plug.Parsers.init(parsers: [:json], json_decoder: Jason)\n    )\n\n  assert conn.body_params == %{\"key\" => \"value\"}\nend)\n```","ref":"readme.html#json-body"},{"type":"extras","title":"URL-encoded body - Sham","doc":"```elixir\nSham.expect(sham, \"POST\", \"/\", fn conn ->\n  conn =\n    Plug.Parsers.call(\n      conn,\n      Plug.Parsers.init(parsers: [:urlencoded], pass: [\"*/*\"])\n    )\n\n  assert conn.body_params == %{\"key\" => \"value\"}\nend)\n```","ref":"readme.html#url-encoded-body"},{"type":"extras","title":"Form-data body - Sham","doc":"```elixir\nSham.expect(sham, \"POST\", \"/\", fn conn ->\n  conn =\n    Plug.Parsers.call(\n      conn,\n      Plug.Parsers.init(parsers: [:multipart], pass: [\"*/*\"])\n    )\n\n  assert conn.body_params == %{\"key\" => \"value\"}\n\n  assert %{\"file\" => %Plug.Upload{path: \"/path/to/file\", filename: \"file.txt\"}} = conn.body_params\nend)\n```","ref":"readme.html#form-data-body"},{"type":"extras","title":"Raw body - Sham","doc":"```elixir\nSham.expect(sham, \"POST\", \"/\", fn conn ->\n  {:ok, body, conn} = Plug.Conn.read_body(conn)\n\n  assert body == \"raw body\"\nend)\n```","ref":"readme.html#raw-body"},{"type":"extras","title":"Generating SSL key and cert for testing - Sham","doc":"```bash\nopenssl req -x509 -nodes -days 3650 -newkey rsa:2048 -keyout test/ssl/key.pem -out test/ssl/cert.pem\n```","ref":"readme.html#generating-ssl-key-and-cert-for-testing"},{"type":"extras","title":"About - Sham","doc":"Thank you for using this library.\n\nIf you’d like to support me, [I am available for Elixir consulting and online pair programming](https://andrewtimberlake.com/hire).","ref":"readme.html#about"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}